"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var Portal_1 = require("./Portal");
var TransparentUnderlay_1 = require("./TransparentUnderlay");
var initialState = {
    isMounting: false,
    isIdle: false,
    isVisible: false,
    isFocused: false,
    isLeaving: false,
    isDismissed: false,
    isTransitioningOut: false,
};
function Postel(props) {
    var timeout = React.useRef(null);
    var toggleRef = React.useRef(undefined);
    var contentRef = React.useRef(undefined);
    var caretRef = React.useRef(undefined);
    var _a = React.useReducer(reducer, initialState), state = _a[0], dispatch = _a[1];
    var _b = React.useState({}), _ = _b[0], forceRender = _b[1];
    React.useEffect(function () {
        function handleReposition() {
            forceRender({});
        }
        window.addEventListener("wheel", handleReposition);
        return function () {
            window.removeEventListener("wheel", handleReposition);
        };
    }, [state.isVisible]);
    var trigger = props.trigger || "hover";
    var triggerDelay = props.triggerDelay || 0;
    var hideTrigger = props.hideTrigger || "mouseleave";
    var hideDelay = props.hideDelay || 0;
    var transitionOutMs = props.transitionOutMs || 0;
    var withLeaveTransition = React.useCallback(function (callback, duration) {
        if (duration === void 0) { duration = 0; }
        clearTimeout(timeout.current);
        if (duration) {
            dispatch({
                type: "TRANSITION_OUT_START",
            });
            timeout.current = setTimeout(function () {
                clearTimeout(timeout.current);
                dispatch({
                    type: "TRANSITION_OUT_COMPLETE",
                });
                callback();
            }, duration);
        }
        else {
            callback();
        }
    }, []);
    var withDelay = React.useCallback(function (callback, delay) {
        if (delay === void 0) { delay = 0; }
        clearTimeout(timeout.current);
        if (delay) {
            timeout.current = setTimeout(function () {
                clearTimeout(timeout.current);
                callback();
            }, delay);
        }
        else {
            callback();
        }
    }, []);
    var withTriggerDelay = React.useCallback(function (callback, delay) {
        if (delay === void 0) { delay = 0; }
        clearTimeout(timeout.current);
        if (state.isVisible && !state.isTransitioningOut) {
            return;
        }
        withDelay(callback, delay);
    }, [state.isVisible, state.isTransitioningOut, withDelay]);
    var handleHide = React.useCallback(function () {
        withDelay(function () {
            withLeaveTransition(function () {
                dispatch({
                    type: "TRIGGER_HIDE",
                });
            }, transitionOutMs);
        }, hideDelay);
    }, [withDelay, withLeaveTransition, transitionOutMs, hideDelay]);
    var onRequestClose = React.useCallback(function () {
        withLeaveTransition(function () {
            dispatch({
                type: "TRIGGER_HIDE",
            });
        }, transitionOutMs);
    }, [transitionOutMs, withLeaveTransition]);
    React.useEffect(function () {
        var _a, _b, _c, _d;
        var toggle = toggleRef.current;
        function handleMouseEnter() {
            withTriggerDelay(function () {
                if (state.isTransitioningOut) {
                    dispatch({
                        type: "TRIGGER_HIDE",
                    });
                }
                dispatch({
                    type: "TRIGGER_SHOW",
                });
            }, triggerDelay);
        }
        function handleClick(event) {
            event.preventDefault();
            event.stopPropagation();
            withTriggerDelay(function () {
                if (state.isTransitioningOut) {
                    dispatch({
                        type: "TRIGGER_HIDE",
                    });
                }
                dispatch({
                    type: "TRIGGER_SHOW",
                });
            }, triggerDelay);
        }
        function handleMouseLeave() {
            handleHide();
        }
        function handleOuterClick(event) {
            var _a, _b, _c;
            var target = event.target;
            var content = contentRef.current;
            var toggle = toggleRef.current;
            var caret = caretRef.current;
            if (((_a = content) === null || _a === void 0 ? void 0 : _a.contains(target)) || ((_b = toggle) === null || _b === void 0 ? void 0 : _b.contains(target)) || ((_c = caret) === null || _c === void 0 ? void 0 : _c.contains(target))) {
                return;
            }
            handleHide();
        }
        if (trigger === "hover") {
            (_a = toggle) === null || _a === void 0 ? void 0 : _a.addEventListener("mouseenter", handleMouseEnter);
        }
        if (trigger === "click") {
            (_b = toggle) === null || _b === void 0 ? void 0 : _b.addEventListener("click", handleClick);
        }
        if (trigger === "mousedown") {
            (_c = toggle) === null || _c === void 0 ? void 0 : _c.addEventListener("mousedown", handleClick);
        }
        if (hideTrigger === "mouseleave") {
            (_d = toggle) === null || _d === void 0 ? void 0 : _d.addEventListener("mouseleave", handleMouseLeave);
        }
        if (hideTrigger === "click") {
            window.addEventListener("click", handleOuterClick);
        }
        return function () {
            var _a, _b, _c, _d;
            (_a = toggle) === null || _a === void 0 ? void 0 : _a.removeEventListener("mouseenter", handleMouseEnter);
            (_b = toggle) === null || _b === void 0 ? void 0 : _b.removeEventListener("mouseleave", handleMouseLeave);
            (_c = toggle) === null || _c === void 0 ? void 0 : _c.removeEventListener("click", handleClick);
            (_d = toggle) === null || _d === void 0 ? void 0 : _d.removeEventListener("mousedown", handleClick);
            window.removeEventListener("click", handleOuterClick);
        };
    }, [
        state.isVisible,
        state.isTransitioningOut,
        trigger,
        triggerDelay,
        hideTrigger,
        transitionOutMs,
        withLeaveTransition,
        withTriggerDelay,
        withDelay,
        handleHide,
    ]);
    function handlePortalMounted() {
        dispatch({
            type: "PORTAL_MOUNTED",
        });
    }
    function getStyles(isVisible, type, placement, preferredAutoPlacement) {
        var defaultStyles = {
            position: "absolute",
            top: 0,
            left: 0,
            transform: "translate3d(0px, 0px, 0px)",
            zIndex: 81,
        };
        var content = contentRef.current;
        var toggle = toggleRef.current;
        var caret = caretRef.current;
        if (!isVisible || !content) {
            if (type === "content") {
                return {
                    style: __assign(__assign({}, defaultStyles), { visibility: "hidden" }),
                };
            }
            if (type === "caret") {
                return {
                    style: __assign(__assign({}, defaultStyles), { visibility: "hidden", transform: "translate3d(0px, 0px, 0px) rotate(45deg)" }),
                };
            }
        }
        if (content && toggle) {
            var position = getPosition(content, toggle, caret, placement, preferredAutoPlacement);
            if (type === "content") {
                return {
                    style: __assign(__assign({}, defaultStyles), { transform: "translate3d(" + position.content.left + "px, " + position.content.top + "px, 0px)" }),
                };
            }
            if (type === "caret") {
                return {
                    style: __assign(__assign({}, defaultStyles), { transformOrigin: position.caret.transform.origin, transform: "translate3d(" + position.caret.left + "px, " + position.caret.top + "px, 0px) rotate(" + position.caret.transform.rotate + ")" }),
                };
            }
        }
    }
    function mapPropsAndRefsToChildren(child, ref, props) {
        return React.cloneElement(child, __assign({ ref: ref }, props));
    }
    var placement = props.placement || "auto";
    var childProps = {
        isTransitioningOut: state.isTransitioningOut,
        onRequestClose: onRequestClose,
    };
    return (React.createElement(React.Fragment, null,
        mapPropsAndRefsToChildren(props.children, toggleRef),
        (state.isVisible || state.isMounting) && props.content && (React.createElement(Portal_1.default, { onReady: handlePortalMounted },
            state.isVisible &&
                !state.isTransitioningOut &&
                props.showTransparentUnderlay && React.createElement(TransparentUnderlay_1.default, null),
            React.createElement("div", __assign({}, getStyles(state.isVisible, "content", placement, props.preferredAutoPlacement)), mapPropsAndRefsToChildren(typeof props.content === "function"
                ? props.content(childProps)
                : props.content, contentRef)),
            props.caret && (React.createElement("div", __assign({}, getStyles(state.isVisible, "caret", placement, props.preferredAutoPlacement)), mapPropsAndRefsToChildren(typeof props.caret === "function"
                ? props.caret(childProps)
                : props.caret, caretRef)))))));
}
exports.default = Postel;
function reducer(state, action) {
    switch (action.type) {
        case "TRIGGER_SHOW": {
            return {
                isMounting: true,
                isIdle: false,
                isFocused: false,
                isLeaving: false,
                isDismissed: false,
                isVisible: false,
                isTransitioningOut: false,
            };
        }
        case "PORTAL_MOUNTED": {
            return {
                isMounting: false,
                isIdle: false,
                isFocused: false,
                isLeaving: false,
                isDismissed: false,
                isVisible: true,
                isTransitioningOut: false,
            };
        }
        case "TRANSITION_OUT_START": {
            return __assign(__assign({}, state), { isTransitioningOut: true });
        }
        case "TRANSITION_OUT_COMPLETE": {
            return __assign(__assign({}, state), { isTransitioningOut: false });
        }
        case "TRIGGER_HIDE": {
            return {
                isMounting: false,
                isIdle: false,
                isFocused: false,
                isLeaving: false,
                isDismissed: true,
                isVisible: false,
                isTransitioningOut: false,
            };
        }
    }
}
function getPosition(content, toggle, caret, preferredPlacement, preferredAutoPlacement, boundingContainer) {
    if (preferredPlacement === void 0) { preferredPlacement = "auto"; }
    var contentRect = getOffset(content);
    var toggleRect = getOffset(toggle);
    var caretRect = caret ? getOffset(caret) : emptyOffset;
    var boundingRect = getOffset(boundingContainer || document.documentElement);
    var top = toggleRect.top - contentRect.height - caretRect.height * 0.5;
    var bottom = toggleRect.top + toggleRect.height + caretRect.height * 0.5;
    var start = toggleRect.left;
    var end = toggleRect.left + toggleRect.width - contentRect.width;
    var center = toggleRect.left + toggleRect.width * 0.5 - contentRect.width * 0.5;
    var verticalCenter = toggleRect.top + toggleRect.height * 0.5 - contentRect.height * 0.5;
    var left = toggleRect.left - contentRect.width - caretRect.width * 0.5;
    var right = toggleRect.left + toggleRect.width + caretRect.width * 0.5;
    var placement = preferredPlacement;
    if (preferredPlacement === "auto") {
        placement =
            preferredAutoPlacement === "auto" || !preferredAutoPlacement
                ? "top"
                : preferredAutoPlacement;
        if (top < boundingRect.scrollY) {
            placement = "bottom";
        }
        if (bottom + contentRect.height - boundingRect.scrollY >
            window.innerHeight) {
            placement = "top";
        }
        var centerWithHorizontalScroll = center - boundingRect.scrollX;
        if (centerWithHorizontalScroll < boundingRect.left &&
            centerWithHorizontalScroll + contentRect.width <= boundingRect.width) {
            placement = placement.concat("-start");
        }
        else if (centerWithHorizontalScroll + contentRect.width > boundingRect.width &&
            centerWithHorizontalScroll >= boundingRect.left) {
            placement = placement.concat("-end");
        }
    }
    var toggleHorizontalCenter = toggleRect.left + toggleRect.width * 0.5;
    var topCaret = {
        top: toggleRect.top - caretRect.height,
        left: toggleHorizontalCenter,
        transform: {
            origin: "0 0",
            rotate: "45deg",
        },
    };
    var bottomCaret = {
        top: toggleRect.top + toggleRect.height,
        left: toggleHorizontalCenter,
        transform: {
            origin: "30% 70%",
            rotate: "-135deg",
        },
    };
    var placements = {
        top: {
            content: { top: top, left: center },
            caret: topCaret,
        },
        bottom: {
            content: { top: bottom, left: center },
            caret: bottomCaret,
        },
        left: {
            content: { top: verticalCenter, left: left },
            caret: {
                top: toggleRect.top + toggleRect.height * 0.5,
                left: toggleRect.left - caretRect.width,
                transform: {
                    origin: "0 0",
                    rotate: "-45deg",
                },
            },
        },
        right: {
            content: { top: verticalCenter, left: right },
            caret: {
                top: toggleRect.top + toggleRect.height * 0.5,
                left: toggleRect.left + toggleRect.width,
                transform: {
                    origin: "70% 30%",
                    rotate: "135deg",
                },
            },
        },
        "top-start": {
            content: { top: top, left: start },
            caret: topCaret,
        },
        "top-end": {
            content: { top: top, left: end },
            caret: topCaret,
        },
        "bottom-start": {
            content: { top: bottom, left: start },
            caret: bottomCaret,
        },
        "bottom-end": {
            content: { top: bottom, left: end },
            caret: bottomCaret,
        },
    };
    return placements[placement];
}
var emptyOffset = {
    top: 0,
    left: 0,
    height: 0,
    width: 0,
    scrollY: 0,
    scrollX: 0,
};
function getOffset(el) {
    var rect = el.getBoundingClientRect(), scrollLeft = window.pageXOffset || document.documentElement.scrollLeft, scrollTop = window.pageYOffset || document.documentElement.scrollTop;
    return {
        top: rect.top + scrollTop,
        left: rect.left + scrollLeft,
        height: rect.height,
        width: rect.width,
        scrollY: window.scrollY,
        scrollX: window.scrollX,
    };
}
//# sourceMappingURL=index.js.map